@@ -5,14 +5,16 @@
  * Copyright (c) 2004-2005 The University of Tennessee and The University
  *                         of Tennessee Research Foundation.  All rights
  *                         reserved.
- * Copyright (c) 2004-2005 High Performance Computing Center Stuttgart, 
+ * Copyright (c) 2004-2005 High Performance Computing Center Stuttgart,
  *                         University of Stuttgart.  All rights reserved.
  * Copyright (c) 2004-2005 The Regents of the University of California.
  *                         All rights reserved.
+ * Copyright (c) 2015      Los Alamos National Security, LLC. All rights
+ *                         reserved.
  * $COPYRIGHT$
- * 
+ *
  * Additional copyrights may follow
- * 
+ *
  * $HEADER$
  */
 /*
@@ -47,9 +49,7 @@
 #include "ompi_config.h"
 
 #include <stdlib.h>
-#ifdef HAVE_STRING_H
 #include <string.h>
-#endif
 #ifdef HAVE_TARGETCONDITIONALS_H
 #include <TargetConditionals.h>
 #endif
@@ -68,6 +68,15 @@ JNIEXPORT jlong JNICALL Java_mpi_Intracomm_split(
     return (jlong)newcomm;
 }
 
+JNIEXPORT jlong JNICALL Java_mpi_Intracomm_splitType(
+        JNIEnv *env, jobject jthis, jlong comm, jint splitType, jint key, jlong info)
+{
+    MPI_Comm newcomm;
+    int rc = MPI_Comm_split_type((MPI_Comm)comm, splitType, key, (MPI_Info)info, &newcomm);
+    ompi_java_exceptionCheck(env, rc);
+    return (jlong)newcomm;
+}
+
 JNIEXPORT jlong JNICALL Java_mpi_Intracomm_create(
         JNIEnv *env, jobject jthis, jlong comm, jlong group)
 {
@@ -77,6 +86,15 @@ JNIEXPORT jlong JNICALL Java_mpi_Intracomm_create(
     return (jlong)newcomm;
 }
 
+JNIEXPORT jlong JNICALL Java_mpi_Intracomm_createGroup(
+        JNIEnv *env, jobject jthis, jlong comm, jlong group, int tag)
+{
+    MPI_Comm newcomm;
+    int rc = MPI_Comm_create_group((MPI_Comm)comm, (MPI_Group)group, tag, &newcomm);
+    ompi_java_exceptionCheck(env, rc);
+    return (jlong)newcomm;
+}
+
 JNIEXPORT jlong JNICALL Java_mpi_Intracomm_createCart(
         JNIEnv *env, jobject jthis, jlong comm,
         jintArray dims, jbooleanArray periods, jboolean reorder)
@@ -129,7 +147,7 @@ JNIEXPORT jlong JNICALL Java_mpi_Intracomm_createDistGraph(
 {
     MPI_Comm graph;
     int nnodes = (*env)->GetArrayLength(env, sources);
-    
+
     jint *jSources, *jDegrees, *jDestins, *jWeights = NULL;
     int  *cSources, *cDegrees, *cDestins, *cWeights = MPI_UNWEIGHTED;
     ompi_java_getIntArray(env, sources, &jSources, &cSources);
@@ -163,12 +181,12 @@ JNIEXPORT jlong JNICALL Java_mpi_Intracomm_createDistGraphAdjacent(
 
     int inDegree  = (*env)->GetArrayLength(env, sources),
         outDegree = (*env)->GetArrayLength(env, destins);
-    
+
     jint *jSources, *jDestins, *jSrcWeights, *jDesWeights;
     int  *cSources, *cDestins, *cSrcWeights, *cDesWeights;
     ompi_java_getIntArray(env, sources, &jSources, &cSources);
     ompi_java_getIntArray(env, destins, &jDestins, &cDestins);
-    
+
     if(weighted)
     {
         ompi_java_getIntArray(env, srcWeights, &jSrcWeights, &cSrcWeights);
@@ -179,7 +197,7 @@ JNIEXPORT jlong JNICALL Java_mpi_Intracomm_createDistGraphAdjacent(
         jSrcWeights = jDesWeights = NULL;
         cSrcWeights = cDesWeights = MPI_UNWEIGHTED;
     }
-    
+
     int rc = MPI_Dist_graph_create_adjacent((MPI_Comm)comm,
              inDegree, cSources, cSrcWeights, outDegree, cDestins,
              cDesWeights, (MPI_Info)info, reorder, &graph);
@@ -219,14 +237,14 @@ JNIEXPORT void JNICALL Java_mpi_Intracomm_scan(
         ompi_java_getReadPtr(&sPtr,&sItem,env,sBuf,sdb,sOff,count,type,bType);
         ompi_java_getWritePtr(&rPtr, &rItem, env, rBuf, rdb, count, type);
     }
-    
+
     MPI_Op op = ompi_java_op_getHandle(env, jOp, hOp, bType);
     int rc = MPI_Scan(sPtr, rPtr, count, type, op, comm);
     ompi_java_exceptionCheck(env, rc);
 
     if(sBuf != NULL)
         ompi_java_releaseReadPtr(sPtr, sItem, sBuf, sdb);
-    
+
     ompi_java_releaseWritePtr(rPtr,rItem,env,rBuf,rdb,rOff,count,type,bType);
 }
 
@@ -340,7 +358,7 @@ JNIEXPORT jlong JNICALL Java_mpi_Intracomm_accept(
                              root, (MPI_Comm)comm, &newComm);
 
     ompi_java_exceptionCheck(env, rc);
-    
+
     if(jport != NULL)
         (*env)->ReleaseStringUTFChars(env, jport, port);
 
